---
title: 'Mutation and update cache'
metaTitle: "Apollo client.mutate for GraphQL mutation delete | GraphQL Android Apollo Tutorial"
metaDescription: "We will use the Apollo client.mutate with variables as an example to delete existing data and update cache locally using read and write methods of apolloStore."
---

import GithubLink from '../../src/GithubLink.js'

Now let's do the integration part. Open `TaskFragment.kt` and add the following code to define the delete mutation

<GithubLink link="https://github.com/hasura/graphql-engine/blob/master/community/learn/graphql-tutorials/tutorials/android-apollo/app-final/app/src/main/java/com/hasura/todo/Todo/ui/todos/TaskFragment.kt" text="TaskFragment.kt" />

```kotlin
    private fun removeTodoMutationCloud(todoId: Int){
        // Init Query
        removeTodoMutation = RemoveTodoMutation.builder().id(todoId).build()

        // Apollo runs query on background thread
        Network.apolloClient.mutate(removeTodoMutation)?.enqueue(object : ApolloCall.Callback<RemoveTodoMutation.Data>() {
            override fun onFailure(error: ApolloException) {
                Log.d("Todo", error.toString() )
            }

            override fun onResponse(@NotNull response: Response<RemoveTodoMutation.Data>) {
                // get data from local cache and update the list
                val index = listItems?.indexOfFirst { todo ->  todo.id() == todoId}
                var todos = (listItems?.toMutableList())?.removeAt(index!!)

                Network.apolloClient
                    .apolloStore()
                    .write(GetMyTodosQuery(), GetMyTodosQuery.Data(mutableListOf(todos))).execute()
                getMyTodosQueryLocal()
            }
        })
    }
```

We have a function defined to handle the onClick of delete icon to remove a todo. Let's update the function.

```kotlin

 override fun delete(taskId: Int) {
-        // Todo : Method for deleting a task
+        removeTodoMutationCloud(taskId)
    }

```
