---
title: "Mutation and update cache"
metaTitle: "Mutation to bulk delete | GraphQL Android Apollo Tutorial"
metaDescription: "Integrate GraphQL Mutation to bulk delete existing personal todos. Update local cache after mutation using apolloStore().read and apolloStore().write methods of apolloClient."
---

import GithubLink from '../../src/GithubLink.js'

Open `TaskFragment.kt` and add the following function to delete all the completed todos from the cloud.

<GithubLink link="https://github.com/hasura/graphql-engine/blob/master/community/learn/graphql-tutorials/tutorials/android-apollo/app-final/app/src/main/java/com/hasura/todo/Todo/ui/todos/TaskFragment.kt" text="TaskFragment.kt" />

```kotlin
private fun removeAllCompletedCloud(){
        // Init Query
        clearCompletedMutation = ClearCompletedMutation.builder().build()

        // Apollo runs query on background thread
        Network.apolloClient.mutate(clearCompletedMutation)?.enqueue(object : ApolloCall.Callback<ClearCompletedMutation.Data>() {
            override fun onFailure(error: ApolloException) {
                Log.d("Todo", error.toString() )
            }

            override fun onResponse(@NotNull response: Response<ClearCompletedMutation.Data>) {
                // get data from local cache and update the list
                val todos = listItems?.filter { task -> task.is_completed }
                Network.apolloClient
                    .apolloStore()
                    .write(GetMyTodosQuery(), GetMyTodosQuery.Data(todos!!)).execute()
                getMyTodosQueryLocal()
            }
        })
    }
```

And finally update the `removeAllCompleted` function as below

```kotlin
  private fun removeAllCompleted() {
-       // Todo : Method for clearing all completed task at once
+       removeAllCompletedCloud()
    }
```

That's a wrap of the basic todo app.
